"""
WSGI config for eztables project.

This module contains the WSGI application used by Django's development server
and any production WSGI deployments. It should expose a module-level variable
named ``application``. Django's ``runserver`` and ``runfcgi`` commands discover
this application via the ``WSGI_APPLICATION`` setting.

Usually you will have the standard Django WSGI application here, but it also
might make sense to replace the whole Django WSGI application with a custom one
that later delegates to the Django one. For example, you could introduce WSGI
middleware here, or combine a Django application with an application of another
framework.

"""
import os

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "eztables.settings")

# This application object is used by any WSGI server configured to use this
# file. This includes Django's development server, if the WSGI_APPLICATION
# setting points here.
from django.core.wsgi import get_wsgi_application
application = get_wsgi_application()

# Apply WSGI middleware here.
# from helloworld.wsgi import HelloWorldApplication
# application = HelloWorldApplication(application)



# -*- coding: utf-8 -*-
from django import forms

ASC = 'asc'
DESC = 'desc'
SORT_DIRS = (
    (ASC, ASC),
    (DESC, DESC),
)


class DatatablesForm(forms.Form):
    '''
    Datatables server side processing Form

    See: http://www.datatables.net/usage/server-side
    '''
    def __init__(self, *args, **kwargs):
        super(DatatablesForm, self).__init__(*args, **kwargs)

        for idx in xrange(int(self.data['iColumns'])):
            self.fields['mDataProp_%s' % idx] = forms.CharField(required=False)
            self.fields['sSearch_%s' % idx] = forms.CharField(required=False)
            self.fields['bRegex_%s' % idx] = forms.BooleanField(required=False)
            self.fields['bSortable_%s' % idx] = forms.BooleanField(required=False)
            self.fields['bSearchable_%s' % idx] = forms.BooleanField(required=False)

        for idx in xrange(int(self.data['iSortingCols'])):
            self.fields['iSortCol_%s' % idx] = forms.IntegerField(required=True)
            self.fields['sSortDir_%s' % idx] = forms.ChoiceField(required=True, choices=SORT_DIRS)

    #: Display start point in the current data set.
    iDisplayStart = forms.IntegerField()

    #: Number of records that the table can display in the current draw.
    #: It is expected that the number of records returned will be equal to this number,
    #: unless the server has fewer records to return.
    iDisplayLength = forms.IntegerField()

    #: Number of columns being displayed (useful for getting individual column search info)
    iColumns = forms.IntegerField()

    #: Global search field
    sSearch = forms.CharField(required=False)

    #: True if the global filter should be treated as a regular expression for advanced filtering, false if not.
    bRegex = forms.BooleanField(required=False)

    #: Number of columns to sort on
    iSortingCols = forms.IntegerField()

    #: Information for DataTables to use for rendering.
    sEcho = forms.CharField()



# -*- coding: utf-8 -*-
from django import template
from djangojs.templatetags.js import js_lib, css


register = template.Library()


@register.simple_tag
def datatables_js():
    return js_lib('datatables/jquery.dataTables.min.js')


@register.simple_tag
def datatables_bootstrap_js():
    return js_lib('datatables/datatables.bootstrap.js')


@register.simple_tag
def datatables_bootstrap_css():
    return css('css/datatables.bootstrap.css')



# -*- coding: utf-8 -*-
from django.views.generic import TemplateView, ListView

from eztables.views import DatatablesView
from eztables.demo.models import Browser


class IndexView(TemplateView):
    template_name = 'eztables/index.html'


class ClientSideView(ListView):
    template_name = 'eztables/client-side.html'
    model = Browser
    context_object_name = 'browsers'


class ServerSideView(TemplateView):
    template_name = 'eztables/server-side-base.html'


class ServerSideSearchView(TemplateView):
    template_name = 'eztables/server-side-search.html'


class ServerSideObjectsView(TemplateView):
    template_name = 'eztables/server-side-objects.html'


class ServerSideCustomView(TemplateView):
    template_name = 'eztables/server-side-custom.html'


class DeferredLoadingView(TemplateView):
    template_name = 'eztables/deferred-loading.html'
    model = Browser
    context_object_name = 'browsers'


class LocalizationView(TemplateView):
    template_name = 'eztables/localization.html'


class BrowserDatatablesView(DatatablesView):
    model = Browser
    fields = (
        'engine__name',
        'name',
        'platform',
        'engine__version',
        'engine__css_grade',
    )


class FormattedBrowserDatatablesView(DatatablesView):
    model = Browser
    fields = (
        'engine__name',
        '{name} {version}',
        'platform',
        'engine__version',
        'engine__css_grade',
    )


class ObjectBrowserDatatablesView(DatatablesView):
    model = Browser
    fields = {
        'name': 'name',
        'engine': 'engine__name',
        'platform': 'platform',
        'engine_version': 'engine__version',
        'css_grade': 'engine__css_grade',
    }


class FormattedObjectBrowserDatatablesView(DatatablesView):
    model = Browser
    fields = {
        'name': '{name} {version}',
        'engine': 'engine__name',
        'platform': 'platform',
        'engine_version': 'engine__version',
        'css_grade': 'engine__css_grade',
    }


class CustomSearchSort(object):

    def sort_col_1(self, direction):
        '''Sort on version instead of name'''
        return '%sversion' % direction

    def sort_col_2(self, direction):
        '''Sort on name and platform instead of platform'''
        return ('%sname' % direction, '%splatform' % direction)

    def search_col_1(self, search, queryset):
        '''Search on version instead of name'''
        return queryset.filter(version__icontains=search)


class CustomBrowserDatatablesView(CustomSearchSort, BrowserDatatablesView):
    pass


class CustomObjectBrowserDatatablesView(CustomSearchSort, ObjectBrowserDatatablesView):
    pass



from django.conf.urls import patterns, include, url

from eztables.demo.views import (
    IndexView,
    ClientSideView,
    ServerSideView,
    ServerSideSearchView,
    ServerSideObjectsView,
    ServerSideCustomView,
    DeferredLoadingView,
    LocalizationView,
    FormattedBrowserDatatablesView,
    ObjectBrowserDatatablesView,
)

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()


urlpatterns = patterns('',
    # Examples:
    url(r'^$', IndexView.as_view(), name='index'),
    url(r'^client-side$', ClientSideView.as_view(), name='client-side'),
    url(r'^server-side$', ServerSideView.as_view(), name='server-side'),
    url(r'^server-side-search$', ServerSideSearchView.as_view(), name='server-side-search'),
    url(r'^server-side-objects$', ServerSideObjectsView.as_view(), name='server-side-objects'),
    url(r'^server-side-custom$', ServerSideCustomView.as_view(), name='server-side-custom'),
    url(r'^defered-loading$', DeferredLoadingView.as_view(), name='deferred-loading'),
    url(r'^localization$', LocalizationView.as_view(), name='localization'),
    url(r'^browsers/', include(patterns('',
        url(r'^default$', FormattedBrowserDatatablesView.as_view(), name='DT-browsers-default'),
        url(r'^objects$', ObjectBrowserDatatablesView.as_view(), name='DT-browsers-objects'),
    ))),
    url(r'^js/', include('djangojs.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    url(r'^admin/', include(admin.site.urls)),
)



# -*- coding: utf-8 -*-
import json
import random
import unittest

from django import forms
from django.conf.urls import patterns, url
from django.core.urlresolvers import reverse
from django.test import TestCase
from factory import Factory, SubFactory, Sequence

from eztables.forms import DatatablesForm
from eztables.views import RE_FORMATTED
from eztables.demo.models import Browser, Engine
from eztables.demo.views import (
    BrowserDatatablesView,
    FormattedBrowserDatatablesView,
    CustomBrowserDatatablesView,
    ObjectBrowserDatatablesView,
    FormattedObjectBrowserDatatablesView,
    CustomObjectBrowserDatatablesView,
)


class EngineFactory(Factory):
    FACTORY_FOR = Engine
    name = random.choice(('Gecko', 'Webkit', 'Presto'))
    version = Sequence(lambda n: n)
    css_grade = random.choice(('A', 'C', 'X'))


class BrowserFactory(Factory):
    FACTORY_FOR = Browser
    name = random.choice(('Firefox', 'Safari', 'Chrome'))
    platform = random.choice(('Windows', 'MacOSX', 'Linux'))
    version = Sequence(lambda n: n)
    engine = SubFactory(EngineFactory)


class DatatablesFormTest(unittest.TestCase):
    def test_base_parameters(self):
        '''Should validate base parameters'''
        form = DatatablesForm({
            'sEcho': '1',
            'iColumns': '5',
            'iDisplayStart': '0',
            'iDisplayLength': '10',
            'sSearch': '',
            'bRegex': 'false',
            'iSortingCols': '1',
            'iSortCol_0': '0',
            'sSortDir_0': 'asc',
        })
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['sEcho'], '1')
        self.assertEqual(form.cleaned_data['iColumns'], 5)
        self.assertEqual(form.cleaned_data['iDisplayStart'], 0)
        self.assertEqual(form.cleaned_data['iDisplayLength'], 10)
        self.assertEqual(form.cleaned_data['sSearch'], '')
        self.assertEqual(form.cleaned_data['bRegex'], False)
        self.assertEqual(form.cleaned_data['iSortingCols'], 1)

    def test_dyanmic_extra_parameters(self):
        '''Should dynamiclly add extra parameters'''
        form = DatatablesForm({
            'sEcho': '1',
            'iColumns': '5',
            'iDisplayStart': '0',
            'iDisplayLength': '10',
            'sSearch': '',
            'bRegex': 'false',
            'iSortingCols': '2',
        })

        for i in xrange(5):
            self.assertTrue('mDataProp_%s' % i in form.fields)
            self.assertTrue(isinstance(form['mDataProp_%s' % i].field, forms.CharField))
            self.assertFalse(form['mDataProp_%s' % i].field.required)

            self.assertTrue('sSearch_%s' % i in form.fields)
            self.assertTrue(isinstance(form['sSearch_%s' % i].field, forms.CharField))
            self.assertFalse(form['sSearch_%s' % i].field.required)

            self.assertTrue('bRegex_%s' % i in form.fields)
            self.assertTrue(isinstance(form['bRegex_%s' % i].field, forms.BooleanField))
            self.assertFalse(form['bRegex_%s' % i].field.required)

            self.assertTrue('bSearchable_%s' % i in form.fields)
            self.assertTrue(isinstance(form['bSearchable_%s' % i].field, forms.BooleanField))
            self.assertFalse(form['bSearchable_%s' % i].field.required)

            self.assertTrue('bSortable_%s' % i in form.fields)
            self.assertTrue(isinstance(form['bSortable_%s' % i].field, forms.BooleanField))
            self.assertFalse(form['bSortable_%s' % i].field.required)

        for i in xrange(2):
            self.assertTrue('iSortCol_%s' % i in form.fields)
            self.assertTrue(isinstance(form['iSortCol_%s' % i].field, forms.IntegerField))
            self.assertTrue(form['iSortCol_%s' % i].field.required)

            self.assertTrue('sSortDir_%s' % i in form.fields)
            self.assertTrue(isinstance(form['sSortDir_%s' % i].field, forms.ChoiceField))
            self.assertTrue(form['sSortDir_%s' % i].field.required)

        self.assertFalse('iSortCol_2' in form.fields)

    def test_valid_extra_parameters(self):
        '''Should validate with extra parameters'''
        form = DatatablesForm({
            'sEcho': '1',
            'iColumns': '5',
            'iDisplayStart': '0',
            'iDisplayLength': '10',
            'sSearch': '',
            'bRegex': 'false',
            'iSortingCols': '1',
            'mDataProp_0': '0',
            'mDataProp_1': '1',
            'mDataProp_2': '2',
            'mDataProp_3': '3',
            'mDataProp_4': '4',
            'sSearch_0': 's0',
            'sSearch_1': 's1',
            'sSearch_2': 's2',
            'sSearch_3': 's3',
            'sSearch_4': 's4',
            'bRegex_0': 'false',
            'bRegex_1': 'false',
            'bRegex_2': 'false',
            'bRegex_3': 'false',
            'bRegex_4': 'false',
            'bSearchable_0': 'true',
            'bSearchable_1': 'true',
            'bSearchable_2': 'true',
            'bSearchable_3': 'true',
            'bSearchable_4': 'true',
            'bSortable_0': 'true',
            'bSortable_1': 'true',
            'bSortable_2': 'true',
            'bSortable_3': 'true',
            'bSortable_4': 'true',
            'iSortCol_0': '0',
            'sSortDir_0': 'asc',
        })
        self.assertTrue(form.is_valid())
        for idx in xrange(5):
            self.assertEqual(form.cleaned_data['mDataProp_%s' % idx], '%s' % idx)
            self.assertEqual(form.cleaned_data['sSearch_%s' % idx], 's%s' % idx)
            self.assertEqual(form.cleaned_data['bRegex_%s' % idx], False)
            self.assertEqual(form.cleaned_data['bSearchable_%s' % idx], True)
            self.assertEqual(form.cleaned_data['bSortable_%s' % idx], True)
        self.assertEqual(form.cleaned_data['iSortCol_0'], 0)
        self.assertEqual(form.cleaned_data['sSortDir_0'], 'asc')

    def test_invalid_sorting_parameters(self):
        '''Should not validate invalid sorting parameters'''
        form = DatatablesForm({
            'sEcho': '1',
            'iColumns': '5',
            'iDisplayStart': '0',
            'iDisplayLength': '10',
            'sSearch': '',
            'bRegex': 'false',
            'iSortingCols': '1',
        })
        self.assertFalse(form.is_valid())


class FormattedFieldRegexTest(unittest.TestCase):
    def test_not_formatted(self):
        '''Should not match unformatted field descriptions'''
        self.assertIsNone(RE_FORMATTED.match('my_field'))

    def test_formatted_single_token(self):
        '''Should match a formatted field description with a single token'''
        matches = RE_FORMATTED.findall('{field}')
        self.assertEqual(len(matches), 1)
        self.assertEqual(matches[0], 'field')

    def test_formatted_multi_token(self):
        '''Should match a formatted field description with a single token'''
        matches = RE_FORMATTED.findall('{field_0}-{field_1}: {field_2}')
        self.assertEqual(len(matches), 3)
        for i in xrange(3):
            self.assertEqual(matches[i], 'field_%s' % i)

    def test_formatted_nester_token(self):
        '''Should match a formatted field description with a single token'''
        matches = RE_FORMATTED.findall('{nested__field_0}-{nested__field_1}: {nested__field_2}')
        self.assertEqual(len(matches), 3)
        for i in xrange(3):
            self.assertEqual(matches[i], 'nested__field_%s' % i)


ENGINE_NAME, NAME, PLATFORM, ENGINE_VERSION, ENGINE_CSS_GRADE = range(5)


class DatatablesTestMixin(object):

    def get_response(self, name, data={}):
        raise NotImplemented

    def value(self, row, field_id):
        raise NotImplemented

    def assertInstance(self, row):
        raise NotImplemented

    def build_query(self, **kwargs):
        query = {
            'sEcho': '1',
            'iColumns': '5',
            'iDisplayStart': '0',
            'iDisplayLength': '10',
            'sSearch': '',
            'bRegex': 'false',
            'mDataProp_0': '0',
            'mDataProp_1': '1',
            'mDataProp_2': '2',
            'mDataProp_3': '3',
            'mDataProp_4': '4',
            'sSearch_0': '',
            'sSearch_1': '',
            'sSearch_2': '',
            'sSearch_3': '',
            'sSearch_4': '',
            'bRegex_0': 'false',
            'bRegex_1': 'false',
            'bRegex_2': 'false',
            'bRegex_3': 'false',
            'bRegex_4': 'false',
            'bSearchable_0': 'true',
            'bSearchable_1': 'true',
            'bSearchable_2': 'true',
            'bSearchable_3': 'true',
            'bSearchable_4': 'true',
            'bSortable_0': 'true',
            'bSortable_1': 'true',
            'bSortable_2': 'true',
            'bSortable_3': 'true',
            'bSortable_4': 'true',
            'iSortingCols': '1',
            'iSortCol_0': '0',
            'sSortDir_0': 'asc',
        }
        query.update(kwargs)
        return query

    def test_empty(self):
        '''Should return an empty Datatables JSON response'''
        response = self.get_response('browsers', self.build_query())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        self.assertTrue('iTotalRecords' in data)
        self.assertEqual(data['iTotalRecords'], 0)
        self.assertTrue('iTotalDisplayRecords' in data)
        self.assertEqual(data['iTotalDisplayRecords'], 0)
        self.assertTrue('sEcho' in data)
        self.assertEqual(data['sEcho'], '1')
        self.assertTrue('aaData' in data)
        self.assertEqual(len(data['aaData']), 0)

    def test_unpaginated(self):
        '''Should return an unpaginated Datatables JSON response'''
        browsers = [BrowserFactory() for _ in xrange(5)]

        response = self.get_response('browsers', self.build_query())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        self.assertTrue('iTotalRecords' in data)
        self.assertEqual(data['iTotalRecords'], len(browsers))
        self.assertTrue('iTotalDisplayRecords' in data)
        self.assertEqual(data['iTotalDisplayRecords'], len(browsers))
        self.assertTrue('sEcho' in data)
        self.assertEqual(data['sEcho'], '1')
        self.assertTrue('aaData' in data)
        self.assertEqual(len(data['aaData']), len(browsers))
        for row in data['aaData']:
            self.assertInstance(row)

    def test_paginated(self):
        '''Should return a paginated Datatables JSON response'''
        browsers = [BrowserFactory() for _ in xrange(15)]

        response = self.get_response('browsers', self.build_query())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        self.assertTrue('iTotalRecords' in data)
        self.assertEqual(data['iTotalRecords'], len(browsers))
        self.assertTrue('iTotalDisplayRecords' in data)
        self.assertEqual(data['iTotalDisplayRecords'], len(browsers))
        self.assertTrue('sEcho' in data)
        self.assertEqual(data['sEcho'], '1')
        self.assertTrue('aaData' in data)
        self.assertEqual(len(data['aaData']), 10)
        for row in data['aaData']:
            self.assertInstance(row)

    def test_formatted(self):
        '''Should return an formatted Datatables JSON response'''
        browsers = [BrowserFactory() for _ in xrange(15)]

        response = self.get_response('formatted-browsers', self.build_query())
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        self.assertTrue('iTotalRecords' in data)
        self.assertEqual(data['iTotalRecords'], len(browsers))
        self.assertTrue('iTotalDisplayRecords' in data)
        self.assertEqual(data['iTotalDisplayRecords'], len(browsers))
        self.assertTrue('sEcho' in data)
        self.assertEqual(data['sEcho'], '1')
        self.assertTrue('aaData' in data)
        self.assertEqual(len(data['aaData']), 10)
        for row in data['aaData']:
            self.assertInstance(row)

    def test_sorted_single_field(self):
        '''Should handle sorting on a single field'''
        for i in xrange(5):
            BrowserFactory(name='Browser %s' % i)

        response = self.get_response('browsers', self.build_query(iSortCol_0=1, sSortDir_0='desc'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        for idx, row in enumerate(data['aaData']):
            field_value = self.value(row, NAME)
            self.assertEqual(field_value, 'Browser %s' % (4 - idx))

    def test_sorted_multiple_field(self):
        '''Should handle sorting on multiple field'''
        for i in xrange(10):
            BrowserFactory(name='Browser %s' % (i / 2), engine__version='%s' % i)

        response = self.get_response('browsers', self.build_query(
            iSortingCols=2,
            iSortCol_0=1,
            sSortDir_0='desc',
            iSortCol_1=3,
            sSortDir_1='asc'
        ))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        expected = (
            ('Browser 4', '8'),
            ('Browser 4', '9'),
            ('Browser 3', '6'),
            ('Browser 3', '7'),
            ('Browser 2', '4'),
            ('Browser 2', '5'),
            ('Browser 1', '2'),
            ('Browser 1', '3'),
            ('Browser 0', '0'),
            ('Browser 0', '1'),
        )
        for idx, row in enumerate(data['aaData']):
            expected_name, expected_version = expected[idx]
            self.assertEqual(self.value(row, NAME), expected_name)
            self.assertEqual(self.value(row, ENGINE_VERSION), expected_version)

    def test_sorted_formatted(self):
        '''Should handle sorting with formatting'''
        for i in xrange(10):
            BrowserFactory(name='Browser %s' % (i / 2), version='%s' % i)

        response = self.get_response('formatted-browsers', self.build_query(iSortCol_0=1, sSortDir_0='desc'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        expected = (
            'Browser 4 9',
            'Browser 4 8',
            'Browser 3 7',
            'Browser 3 6',
            'Browser 2 5',
            'Browser 2 4',
            'Browser 1 3',
            'Browser 1 2',
            'Browser 0 1',
            'Browser 0 0',
        )
        for idx, row in enumerate(data['aaData']):
            self.assertEqual(self.value(row, NAME), expected[idx])

    def test_sorted_custom_implementation(self):
        '''Should handle sorting with a custom implementation'''
        for i in xrange(5):
            BrowserFactory(name='Browser %s' % i, version='%s' % (5 - i))

        response = self.get_response('custom-browsers', self.build_query(iSortCol_0=1, sSortDir_0='desc'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        for idx, row in enumerate(data['aaData']):
            field_value = self.value(row, NAME)
            self.assertEqual(field_value, 'Browser %s' % idx)

    def test_sorted_custom_implementation_many_fields(self):
        '''Should handle sorting with a custom implementation on many fields'''
        for i in xrange(5):
            BrowserFactory(name='Browser %s' % i, platform='%s' % (5 - i))

        response = self.get_response('custom-browsers', self.build_query(iSortCol_0=2, sSortDir_0='asc'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'application/json')

        data = json.loads(response.content)
        for idx, row in enumerate(data['aaData']):
            field_value = self.value(row, NAME)
            self.assertEqual(field_value, 'Browser %s' % idx)

    def test_global_search_single_term(self):
        '''Should do a global search on a single term'''
        for _ in xrange(2):
            BrowserFactory(name='test')
        for _ in xrange(3):
            BrowserFactory(engine__name='engine')

        response = self.get_response('browsers', self.build_query(sSearch='test'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 2)
        for row in data['aaData']:
            self.assertEqual(self.value(row, NAME), 'test')

        response = self.get_response('browsers', self.build_query(sSearch='engine'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 3)
        for row in data['aaData']:
            self.assertEqual(self.value(row, ENGINE_NAME), 'engine')

    def test_global_search_many_terms(self):
        '''Should do a global search on many terms'''
        for _ in xrange(2):
            BrowserFactory(name='test')
        for _ in xrange(3):
            BrowserFactory(engine__name='engine')
        for _ in xrange(4):
            BrowserFactory(name='test', engine__name='engine')

        response = self.get_response('browsers', self.build_query(sSearch='test engine'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 4)
        for row in data['aaData']:
            self.assertEqual(self.value(row, ENGINE_NAME), 'engine')
            self.assertEqual(self.value(row, NAME), 'test')

    def test_global_search_regex(self):
        '''Should do a global search on a regex'''
        for _ in xrange(3):
            BrowserFactory()
        for _ in xrange(2):
            BrowserFactory(name='test')

        response = self.get_response('browsers', self.build_query(sSearch='[tes]{4}', bRegex=True))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 2)
        for row in data['aaData']:
            self.assertEqual(self.value(row, NAME), 'test')

    def test_column_search_single_column(self):
        '''Should filter on a single column'''
        for _ in xrange(3):
            BrowserFactory()
        for _ in xrange(2):
            BrowserFactory(name='test')

        response = self.get_response('browsers', self.build_query(sSearch_1='tes'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 2)
        for row in data['aaData']:
            self.assertEqual(self.value(row, NAME), 'test')

    def test_column_search_many_columns(self):
        '''Should filter on many columns'''
        for _ in xrange(2):
            BrowserFactory(name='test')
        for _ in xrange(3):
            BrowserFactory(engine__name='engine')
        for _ in xrange(4):
            BrowserFactory(name='test', engine__name='engine')

        response = self.get_response('browsers', self.build_query(sSearch_0='eng', sSearch_1='tes'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 4)
        for row in data['aaData']:
            self.assertEqual(self.value(row, ENGINE_NAME), 'engine')
            self.assertEqual(self.value(row, NAME), 'test')

    def test_column_search_formatted_column(self):
        '''Should filter on a formatted column'''
        for _ in xrange(3):
            BrowserFactory()
        for _ in xrange(2):
            BrowserFactory(name='test')

        response = self.get_response('formatted-browsers', self.build_query(sSearch_1='tes'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 2)
        for row in data['aaData']:
            self.assertTrue(self.value(row, NAME).startswith('test'))

    def test_column_search_regex(self):
        '''Should filter on a single column with regex'''
        for _ in xrange(3):
            BrowserFactory()
        for _ in xrange(2):
            BrowserFactory(name='test')

        response = self.get_response('browsers', self.build_query(sSearch_1='[tes]{4}', bRegex_1=True))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 2)
        for row in data['aaData']:
            self.assertEqual(self.value(row, NAME), 'test')

    def test_column_search_custom(self):
        '''Should filter on a single column with custom search'''
        for i in xrange(3):
            BrowserFactory(version='%s.%s' % (i, i))
        for _ in xrange(2):
            BrowserFactory(name='test')

        response = self.get_response('custom-browsers', self.build_query(sSearch_1='tes'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 0)
        for row in data['aaData']:
            self.assertEqual(self.value(row, NAME), 'test')

        response = self.get_response('custom-browsers', self.build_query(sSearch_1='1.1'))
        data = json.loads(response.content)
        self.assertEqual(len(data['aaData']), 1)
        for row in data['aaData']:
            self.assertNotEqual(self.value(row, NAME), 'test')


class ArrayMixin(object):
    urls = patterns('',
        url(r'^$', BrowserDatatablesView.as_view(), name='browsers'),
        url(r'^formatted/$', FormattedBrowserDatatablesView.as_view(), name='formatted-browsers'),
        url(r'^custom/$', CustomBrowserDatatablesView.as_view(), name='custom-browsers'),
    )

    def value(self, row, field_id):
        return row[field_id]

    def assertInstance(self, row):
        self.assertTrue(isinstance(row, list))
        self.assertEqual(len(row), 5)


class ObjectMixin(object):
    urls = patterns('',
        url(r'^$', ObjectBrowserDatatablesView.as_view(), name='browsers'),
        url(r'^formatted/$', FormattedObjectBrowserDatatablesView.as_view(), name='formatted-browsers'),
        url(r'^custom/$', CustomObjectBrowserDatatablesView.as_view(), name='custom-browsers'),
    )

    id_to_name = {
        0: 'engine',
        1: 'name',
        2: 'platform',
        3: 'engine_version',
        5: 'css_grade',
    }

    def build_query(self, **kwargs):
        query = super(ObjectMixin, self).build_query(**kwargs)
        query.update(dict((
            ('mDataProp_%s' % k, v) for k, v in self.id_to_name.iteritems()
        )))
        return query

    def value(self, row, field_id):
        return row[self.id_to_name[field_id]]

    def assertInstance(self, row):
        self.assertTrue(isinstance(row, dict))
        for key in self.id_to_name.values():
            self.assertTrue(key in row)


class GetMixin(object):
    def get_response(self, name, data={}):
        return self.client.get(reverse(name), data)


class PostMixin(object):
    def get_response(self, name, data={}):
        return self.client.post(reverse(name), data)


class DatatablesArrayGetTest(ArrayMixin, GetMixin, DatatablesTestMixin, TestCase):
    pass


class DatatablesArrayPostTest(ArrayMixin, PostMixin, DatatablesTestMixin, TestCase):
    pass


class DatatablesObjGetTest(ObjectMixin, GetMixin, DatatablesTestMixin, TestCase):
    pass


class DatatablesObjPostTest(ObjectMixin, PostMixin, DatatablesTestMixin, TestCase):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re

from os.path import join
from setuptools import setup, find_packages

RE_REQUIREMENT = re.compile(r'^\s*-r\s*(?P<filename>.*)$')


def pip(filename):
    '''Parse pip requirement file and transform it to setuptools requirements'''
    requirements = []
    for line in open(join('requirements', filename)).readlines():
        match = RE_REQUIREMENT.match(line)
        if match:
            requirements.extend(pip(match.group('filename')))
        else:
            requirements.append(line)
    return requirements


def rst(filename):
    '''
    Load rst file and sanitize it for PyPI.
    Remove unsupported github tags:
     - code-block directive
    '''
    content = open(filename).read()
    return re.sub(r'\.\.\s? code-block::\s*(\w|\+)+', '::', content)


long_description = '\n'.join((
    rst('README.rst'),
    rst('CHANGELOG.rst'),
    ''
))

setup(
    name='django-eztables',
    version=__import__('eztables').__version__,
    description=__import__('eztables').__description__,
    long_description=long_description,
    url='https://github.com/noirbizarre/django-eztables',
    download_url='http://pypi.python.org/pypi/django-eztables',
    author='Axel Haustant',
    author_email='noirbizarre+django@gmail.com',
    packages=find_packages(),
    include_package_data=True,
    install_requires=pip('install.pip'),
    tests_require=pip('develop.pip'),
    license='LGPL',
    classifiers=[
        "Framework :: Django",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: System :: Software Distribution",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
    ],
)



# -*- coding: utf-8 -*-
import json
import re

from operator import or_

from django.core.paginator import Paginator
from django.core.serializers.json import DjangoJSONEncoder
from django.db.models import Q
from django.http import HttpResponse, HttpResponseBadRequest
from django.views.generic import View
from django.views.generic.list import MultipleObjectMixin

from eztables.forms import DatatablesForm, DESC


JSON_MIMETYPE = 'application/json'

RE_FORMATTED = re.compile(r'\{(\w+)\}')


class DatatablesView(MultipleObjectMixin, View):
    '''
    Render a paginated server-side Datatables JSON view.

    See: http://www.datatables.net/usage/server-side
    '''
    fields = []
    _db_fields = None

    def post(self, request, *args, **kwargs):
        return self.process_dt_response(request.POST)

    def get(self, request, *args, **kwargs):
        return self.process_dt_response(request.GET)

    def process_dt_response(self, data):
        self.form = DatatablesForm(data)
        if self.form.is_valid():
            self.object_list = self.get_queryset().values(*self.get_db_fields())
            return self.render_to_response(self.form)
        else:
            return HttpResponseBadRequest()

    def get_db_fields(self):
        if not self._db_fields:
            self._db_fields = []
            fields = self.fields.values() if isinstance(self.fields, dict) else self.fields
            for field in fields:
                if RE_FORMATTED.match(field):
                    self._db_fields.extend(RE_FORMATTED.findall(field))
                else:
                    self._db_fields.append(field)
        return self._db_fields

    @property
    def dt_data(self):
        return self.form.cleaned_data

    def get_field(self, index):
        if isinstance(self.fields, dict):
            return self.fields[self.dt_data['mDataProp_%s' % index]]
        else:
            return self.fields[index]

    def get_orders(self):
        '''Get ordering fields for ``QuerySet.order_by``'''
        orders = []
        iSortingCols = self.dt_data['iSortingCols']
        dt_orders = [(self.dt_data['iSortCol_%s' % i], self.dt_data['sSortDir_%s' % i]) for i in xrange(iSortingCols)]
        for field_idx, field_dir in dt_orders:
            direction = '-' if field_dir == DESC else ''
            if hasattr(self, 'sort_col_%s' % field_idx):
                method = getattr(self, 'sort_col_%s' % field_idx)
                result = method(direction)
                if isinstance(result, str):
                    orders.append(result)
                else:
                    orders.extend(result)
            else:
                field = self.get_field(field_idx)
                if RE_FORMATTED.match(field):
                    tokens = RE_FORMATTED.findall(field)
                    orders.extend(['%s%s' % (direction, token) for token in tokens])
                else:
                    orders.append('%s%s' % (direction, field))
        return orders

    def global_search(self, queryset):
        '''Filter a queryset with global search'''
        search = self.dt_data['sSearch']
        if search:
            if self.dt_data['bRegex']:
                criterions = (Q(**{'%s__iregex' % field: search}) for field in self.get_db_fields())
                search = reduce(or_, criterions)
                queryset = queryset.filter(search)
            else:
                for term in search.split():
                    criterions = (Q(**{'%s__icontains' % field: term}) for field in self.get_db_fields())
                    search = reduce(or_, criterions)
                    queryset = queryset.filter(search)
        return queryset

    def column_search(self, queryset):
        '''Filter a queryset with column search'''
        for idx in xrange(self.dt_data['iColumns']):
            search = self.dt_data['sSearch_%s' % idx]
            if search:
                if hasattr(self, 'search_col_%s' % idx):
                    custom_search = getattr(self, 'search_col_%s' % idx)
                    queryset = custom_search(search, queryset)
                else:
                    field = self.get_field(idx)
                    fields = RE_FORMATTED.findall(field) if RE_FORMATTED.match(field) else [field]
                    if self.dt_data['bRegex_%s' % idx]:
                        criterions = (Q(**{'%s__iregex' % field: search}) for field in fields)
                        search = reduce(or_, criterions)
                        queryset = queryset.filter(search)
                    else:
                        for term in search.split():
                            criterions = (Q(**{'%s__icontains' % field: term}) for field in fields)
                            search = reduce(or_, criterions)
                            queryset = queryset.filter(search)
        return queryset

    def get_queryset(self):
        '''Apply Datatables sort and search criterion to QuerySet'''
        qs = super(DatatablesView, self).get_queryset()
        # Perform global search
        qs = self.global_search(qs)
        # Perform column search
        qs = self.column_search(qs)
        # Return the ordered queryset
        return qs.order_by(*self.get_orders())

    def get_page(self, form):
        '''Get the requested page'''
        page_size = form.cleaned_data['iDisplayLength']
        start_index = form.cleaned_data['iDisplayStart']
        paginator = Paginator(self.object_list, page_size)
        num_page = (start_index / page_size) + 1
        return paginator.page(num_page)

    def get_rows(self, rows):
        '''Format all rows'''
        return [self.get_row(row) for row in rows]

    def get_row(self, row):
        '''Format a single row (if necessary)'''
        if isinstance(self.fields, dict):
            return dict([
                (key, value.format(**row) if RE_FORMATTED.match(value) else row[value])
                for key, value in self.fields.items()
            ])
        else:
            return [field.format(**row) if RE_FORMATTED.match(field) else row[field] for field in self.fields]

    def render_to_response(self, form, **kwargs):
        '''Render Datatables expected JSON format'''
        page = self.get_page(form)
        data = {
            'iTotalRecords': page.paginator.count,
            'iTotalDisplayRecords': page.paginator.count,
            'sEcho': form.cleaned_data['sEcho'],
            'aaData': self.get_rows(page.object_list),
        }
        return self.json_response(data)

    def json_response(self, data):
        return HttpResponse(
            json.dumps(data, cls=DjangoJSONEncoder),
            mimetype=JSON_MIMETYPE
        )
